// This file was generated by gir (https://github.com/gtk-rs/gir)
// from
// from gir-files (https://github.com/gtk-rs/gir-files)
// DO NOT EDIT

use crate::ffi;
use glib::GStr;

#[doc(alias = "SOUP_FORM_MIME_TYPE_MULTIPART")]
pub static FORM_MIME_TYPE_MULTIPART: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::SOUP_FORM_MIME_TYPE_MULTIPART) };
#[doc(alias = "SOUP_FORM_MIME_TYPE_URLENCODED")]
pub static FORM_MIME_TYPE_URLENCODED: &GStr =
    unsafe { GStr::from_utf8_with_nul_unchecked(ffi::SOUP_FORM_MIME_TYPE_URLENCODED) };
