// This file was generated by gir (https://github.com/gtk-rs/gir)
// from
// from gir-files (https://github.com/gtk-rs/gir-files.git)
// DO NOT EDIT

use crate::{ffi, AnimationTarget};
use glib::{prelude::*, translate::*};

glib::wrapper! {
    #[doc(alias = "AdwNoneAnimationTarget")]
    pub struct NoneAnimationTarget(Object<ffi::AdwNoneAnimationTarget, ffi::AdwNoneAnimationTargetClass>) @extends AnimationTarget;

    match fn {
        type_ => || ffi::adw_none_animation_target_get_type(),
    }
}

impl NoneAnimationTarget {
    #[doc(alias = "adw_none_animation_target_new")]
    pub fn new() -> NoneAnimationTarget {
        assert_initialized_main_thread!();
        unsafe {
            AnimationTarget::from_glib_full(ffi::adw_none_animation_target_new()).unsafe_cast()
        }
    }
}

#[cfg(feature = "v1_9")]
#[cfg_attr(docsrs, doc(cfg(feature = "v1_9")))]
impl Default for NoneAnimationTarget {
    fn default() -> Self {
        Self::new()
    }
}
