// This file was generated by gir (https://github.com/gtk-rs/gir)
// from
// from gir-files (https://github.com/gtk-rs/gir-files)
// DO NOT EDIT

use crate::{ffi, WebsocketExtension};

glib::wrapper! {
    #[doc(alias = "SoupWebsocketExtensionDeflate")]
    pub struct WebsocketExtensionDeflate(Object<ffi::SoupWebsocketExtensionDeflate, ffi::SoupWebsocketExtensionDeflateClass>) @extends WebsocketExtension;

    match fn {
        type_ => || ffi::soup_websocket_extension_deflate_get_type(),
    }
}

impl WebsocketExtensionDeflate {}
